﻿using System;
using System.Linq;
using System.Web;
using Dynamicweb.eCommerce.Orders;
using Dynamicweb.eCommerce.Orders.SalesDiscounts;
using Dynamicweb.Extensibility;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Samples.Lib.IPService;

namespace Dynamicweb.Samples.Lib
{
	/// <summary>
	/// Implements a SalesDiscountProvider that gives a discount to users from
	/// specific countries (determioned by their IP address).
	/// </summary>
	[AddInName("Discount for specific countries")]
	[AddInDescription(@"This discount applies when the user's IP is located in one of 
			the assigned countries. Enter one or more comma separated country codes to which 
			the discount applies.")]
	public class IPBasedDiscountProvider : SalesDiscountProvider
	{
		private const string percentageSessionKey = "OutsideTemperatureProvider_DiscountPercentage";

		
		/// <summary>
		/// Gets or sets the applicable country codes. Should be a comma separated list of two letter country
		/// codes such as NL or GB.
		/// </summary>
		/// <value>The applicable country codes.</value>
		[AddInParameter("Country Codes"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string ApplicableCountryCodes { get; set; }

		/// <summary>
		/// Initializes a new instance of the IPBasedDiscountProvider class.
		/// Assigns a default of NL to ApplicableCountryCodes.
		/// </summary>
		public IPBasedDiscountProvider()
		{
			ApplicableCountryCodes = "NL";
		}

		/// <summary>
		/// Processes the order and calculates a discount if applicable.
		/// </summary>
		public override void ProcessOrder(eCommerce.Orders.Order order)
		{
			if (DiscountValue.Type == DiscountTypes.Percent)
			{
				double? discountPercentage = GetPercentageFromSession();
				if (!discountPercentage.HasValue)
				{
					discountPercentage = GetPercentageFromWebService();
					StorePercentageInSession(discountPercentage);
				}

				if (discountPercentage > 0)
				{
					double discountPrice = CalculateDiscountPrice(order, discountPercentage);

					OrderLine line = CreateOrderLine(order, discountPrice);

					// Add the order line to the order
					order.OrderLines.Add(line, false);
				}
			}
			else
			{
				throw new NotImplementedException(@"Discounts for types other than Percent 
                            are not supported in this demo.");
			}
		}

		/// <summary>
		/// Creates a discount order line with <paramref name="discountPrice"/> as the value.
		/// </summary>
		private OrderLine CreateOrderLine(eCommerce.Orders.Order order, double discountPrice)
		{
			OrderLine line = new OrderLine { Order = order, Quantity = 1, ProductName = DiscountName };
			line.SetUnitPrice(discountPrice);
			line.Type = Base.ChkString(Base.ChkNumber(OrderLine.OrderLineType.Discount));
			return line;
		}

		/// <summary>
		/// Calculates the discount price.
		/// </summary>
		private static double CalculateDiscountPrice(eCommerce.Orders.Order order, double? discountPercentage)
		{
			double discountPrice = (order.PriceBeforeFees.Price / 100) * discountPercentage.Value;
			discountPrice = discountPrice * -1;
			return discountPrice;
		}

		/// <summary>
		/// Gets the percentage from session state if it exists. Returns null if not found.
		/// </summary>
		private static double? GetPercentageFromSession()
		{
			return HttpContext.Current.Session[percentageSessionKey] as double?;
		}

		/// <summary>
		/// Stores the percentage in session state.
		/// </summary>
		private static void StorePercentageInSession(double? discountPercentage)
		{
			HttpContext.Current.Session[percentageSessionKey] = discountPercentage;
		}

		/// <summary>
		/// Gets a discount percentage based on the IIP address of the customer by calling into an external web service.
		/// Makes use of this public service: http://www.webservicex.net/WCF/ServiceDetails.aspx?SID=46
		/// </summary>
		private double GetPercentageFromWebService()
		{
			try
			{
				using (IPService.GeoIPService myService = new IPService.GeoIPService())
				{
					string ipAddress = HttpContext.Current.Request.ServerVariables["remote_addr"];
					GeoIP result = myService.GetGeoIP(ipAddress);
					bool isMatch = (from c in ApplicableCountryCodes.Split(new char[] { ',' }) 
													where c.Trim().ToLower() == result.CountryCode.ToLower() 
													select c).Count() > 0;
					return isMatch ? DiscountValue.Amount : 0;
				}
			}
			catch
			{
				return 0;
			}
		}
	}
}
